% A Graphical User Interface for Constraint Logic Programming
% http://contraintes.inria.fr/~fages/CLPGUI
% Copyright (C) 2001,2002 INRIA Francois Fages
% License LGPL-2.1

% Version 2.1.3
% sicstus.pl 
% Sicstus Prolog compatibility file for CLPGUI
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%:- module(gui, [
%		gui/0,
%		gui/1,
%		gui_varnames/2,
%		gui_varnames/1,
%		gui_addvarnames/2,
%		gui_varnames/0,
%		gui_show_sizes/0,
%		gui_show_intervals/0,
%		gui_show_values/0,
%		gui_show_no_domain/0,
%		gui_button/1,
%		gui_bagof_buttons/2,
%		gui_trace_search/1,
%		gui_trace_call/1,
%		gui_trace_search_predicate/1,
%		gui_trace_call_predicate/1,
%		gui_trace_labeling/1,
%		gui_show/0,
%		gui_show/1,
%		gui_interact/0,
%		gui_no_trace_backtracking/0,
%		gui_trace_backtracking/0
%	       ]).

% Examples
%%%%%%%%%%
:- include('../examplesSicstus/append.pl').
:- include('../examplesSicstus/bridge.pl').
:- include('../examplesSicstus/smm.pl').
:- include('../examplesSicstus/sudoku.pl').


% Writing finite domains
%%%%%%%%%%%%%%%%%%%%%%%%
:- use_module(library(clpfd)).

generic_var(X):- var(X).

gui_fd(X):- catch((fd_size(X,S), S\=sup),_,fail).

fd_labeling(X):- indomain(X).

gui_print_dom(X):-
    fd_size(X,S),
    gui_printnf(S),
    (S=1
     ->
     gui_print(X)
     ;
     fd_dom(X,D),
     write(output_stream,'#_('), % GNU-Prolog representation of finite domains
     gui_print_domain(D),
     gui_print(')')).

gui_print_domain(I):- number(I), !, write(output_stream,I).
gui_print_domain({I}):- !, write(output_stream,I).
gui_print_domain(A\/B):- !,
    gui_print_domain(A), write(output_stream,':'), gui_print_domain(B).
gui_print_domain((A..B)):- !,
    gui_print_domain(A), write(output_stream,'..'), gui_print_domain(B).

% Errors
%%%%%%%%
gui_catch_error(G,X,E):-
	catch(G,
	      X,
	      (
		functor(X,F,_),
		((F=instanciation_error;F=type_error;F=domain_error;
		  F=existence_error;F=permission_error;F=context_error;
		  F=syntax_error;F=evaluation_error;F=representation_error;
		  F=consistency_error;F=ressource_error;F=system_error)
		->
		    call(E)
		;
		    throw(X)))).

% Misc
%%%%%%
:- use_module(library(lists)).
:- use_module(library(system)).

exec(C):- exec(C,[std,std,std],_).

number_atom(I,A):-
    (number(I)
     ->
     number_codes(I,C), atom_codes(A,C)
     ;
     atom_codes(A,C), number_codes(I,C)).     


% Sockets
%%%%%%%%%
:- use_module(library(sockets)).

socket_accept(S,C,I,O):-
    I=O,
    socket_accept(S,C,I).

% Stream aliases
%%%%%%%%%%%%%%%%
add_stream_alias(S,A):-
    stream_code(S, Code),
    prolog:assert_stream_properties([A], [], Code).


% Command line arguments
%%%%%%%%%%%%%%%%%%%%%%%%
argument_counter(N):-
    prolog_flag(argv,L),
    length(L,N).

argument_value(N,A):-
    prolog_flag(argv,L),
    nth(N,L,A).
    
% Reading terms
%%%%%%%%%%%%%%%
name_query_vars(L,_):- name_query_vars(L).
name_query_vars([]).
name_query_vars([(N='$VARNAME'(N))|L]):-
	name_query_vars(L).

portray('$VARNAME'(N)):-print(output_stream,N).

% GNU-Prolog global variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

g_assign(X,V):-
    bb_put(X,V).

g_link(X,V):-
    (bb_get(X,U) -> true ; U=0),
    (U==g_link
    ->
     update_global_mutable(X,V)
    ;
     undo(user:g_assign(X,U)),
     g_assign(X,g_link),
     create_global_mutable(X,V)).

g_read(X,V):-
    (bb_get(X,U) 
    ->
      (U==g_link
      ->
       get_global_mutable(X,V)
      ;
       V=U) 
    ;
      V=0).

%% using Mats Carlsson's global mutables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% create_global_mutable(+Name, -Value):
%% A new entry with functor Name/1 is added to the global
%% private slot. Its argument is a new mutable, the value of
%% which is Value.  
create_global_mutable(Name, Value) :-
	execution_state(break_level(0), private(Priv)),
	Mem =.. [Name, Mutable],
	memberchk(Mem, Priv),
	create_mutable(Value, Mutable).

%% get_global_mutable(+Name, -Value):
%% The entry with functor Name/1 should already exists in the global
%% private slot. Its argument is assumed to be a mutable, the value of
%% which is unified with Value.  
get_global_mutable(Name, Value) :-
	execution_state(break_level(0), private(Priv)),
	Mem =.. [Name, Mutable],
	memberchk(Mem, Priv),
	get_mutable(Value, Mutable).

%% update_global_mutable(+Name, -Value):
%% The entry with functor Name/1 should already exists in the global
%% private slot. Its argument is assumed to be a mutable, the value of
%% which is updated to Value.  
update_global_mutable(Name, Value) :-
	execution_state(break_level(0), private(Priv)),
	Mem =.. [Name, Mutable],
	memberchk(Mem, Priv),
	update_mutable(Value, Mutable).

% Predicate pathname
%%%%%%%%%%%%%%%%%%%%

:- prolog_load_context(directory,F), write(F),nl, g_assign(predicate_pathname,F).

predicate_pathname(_,P):- 
   write(search),nl,
   g_read(predicate_pathname,F),
   atom_concat(F,'/clpguiSicstus.pl',P).



